@@ -20,6 +20,7 @@
 #include <microfeed-common/microfeedconfiguration.h>
 #include <microfeed-common/microfeedmisc.h>
 #include <microfeed-common/microfeedstore.h>
+#include <microfeed-common/microfeedprotocol.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -106,7 +107,7 @@ MicrofeedConfiguration* microfeed_configuration_new(void) {
 		memcpy(configuration->wizards_home_directory, directory, length);
 		memcpy(configuration->wizards_home_directory + length, "/.microfeed/wizards", 20); /* Remember the NULL byte */
 	}
-	configuration->subscriptions = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)subscription_get_application_identifier);
+	configuration->subscriptions = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)subscription_get_application_identifier);
 	
 	return configuration;
 }
@@ -488,10 +489,15 @@ static void get_publishers(MicrofeedConfiguration* configuration, const char* di
 	struct dirent* dirent;
 	int i;
 	int order;
+	char* left_separator;
+	char* right_separator;
 	
 	if ((dir = opendir(directory))) {
 		while ((dirent = readdir(dir))) {
-			if (dirent->d_name[0] != '.') {
+			if (dirent->d_name[0] != '.' &&
+			    (left_separator = strchr(dirent->d_name, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR)) &&
+			    (right_separator = strrchr(dirent->d_name, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR)) &&
+			    right_separator == left_separator) {
 				for (i = 0; i < *length_pointer; i++) {
 					if ((order = strcmp(configuration->publishers[i], dirent->d_name)) >= 0) {
 						break;
