@@ -26,13 +26,11 @@
 #include <inttypes.h>
 #include <time.h>
 #include <stdio.h>
+#include <arpa/inet.h>
 
 typedef uint16_t property_size_t;
 
 struct _MicrofeedItem {
-	char* uid;
-	time_t timestamp;
-	MicrofeedItemStatus status;
 	size_t* properties; /* Offsets from the start of the buffer */
 	unsigned int properties_length;
 	unsigned int properties_reserved;
@@ -49,16 +47,34 @@ struct _MicrofeedItemIterator {
 	unsigned int index;
 };
 
+typedef struct {
+	char status;
+	property_size_t uid_length;
+	uint32_t timestamp;
+	char uid[1];
+} Buffer;
+
 static int get_index(const MicrofeedItem* item, const char* key, size_t length, unsigned int* index);
+
+static MicrofeedItem* item_new(const char* uid) {
+	MicrofeedItem* item;
+	size_t length;
+	
+	length = strlen(uid);
+	item = microfeed_memory_allocate(MicrofeedItem);
+	item->buffer_size = sizeof(Buffer) + length;
+	item->buffer = (char*)microfeed_memory_allocate_bytes(item->buffer_size);
+	((Buffer*)item->buffer)->uid_length = length;
+	memcpy(((Buffer*)item->buffer)->uid, uid, length + 1);
+
+	return item;
+}
 	
 MicrofeedItem* microfeed_item_new(const char* uid, time_t timestamp) {
 	MicrofeedItem* item;
 	
-	item = microfeed_memory_allocate(MicrofeedItem);
-	if (uid) {
-		item->uid = strdup(uid);
-	}
-	item->timestamp = timestamp;
+	item = item_new(uid);
+	microfeed_item_set_timestamp(item, timestamp);
 
 	return item;
 }
@@ -66,9 +82,8 @@ MicrofeedItem* microfeed_item_new(const char* uid, time_t timestamp) {
 MicrofeedItem* microfeed_item_new_temporary(void) {
 	MicrofeedItem* item;
 
-	item = microfeed_memory_allocate(MicrofeedItem);
-	item->uid = microfeed_util_create_unique_identifier();
-	item->timestamp = time(NULL);
+	item = item_new(microfeed_util_create_unique_identifier());
+	microfeed_item_set_timestamp(item, time(NULL));
 
 	return item;
 }
@@ -76,18 +91,53 @@ MicrofeedItem* microfeed_item_new_temporary(void) {
 MicrofeedItem* microfeed_item_new_with_status(const char* uid, time_t timestamp, MicrofeedItemStatus status) {
 	MicrofeedItem* item;
 	
-	item = microfeed_item_new(uid, timestamp);
-	item->status = status | MICROFEED_ITEM_STATUS_SET;
+	item = item_new(uid);
+	microfeed_item_set_timestamp(item, timestamp);
+	microfeed_item_set_status(item, status);
 	
 	return item;
 }
 
+MicrofeedItem* microfeed_item_new_from_marshalled(const void* data, size_t size) {
+	MicrofeedItem* item = NULL;
+	size_t offset;
+
+	if (size > sizeof(Buffer)) {
+		item = microfeed_memory_allocate(MicrofeedItem);
+		item->buffer = (char*)malloc(size);
+		memcpy(item->buffer, data, size);
+		item->buffer_size = size;
+
+		for (offset = sizeof(Buffer) + ((Buffer*)item->buffer)->uid_length; offset < size; offset += *((property_size_t*)(item->buffer + offset))) {
+			if (item->properties_length == item->properties_reserved) {
+				item->properties_reserved += 8 * sizeof(size_t);
+				item->properties = (size_t*)realloc(item->properties, item->properties_reserved * sizeof(size_t));
+			}
+			item->properties[item->properties_length++] = offset;
+		}
+
+		if (offset > size) {
+			free(item->buffer);
+			item->buffer = NULL;
+			item->buffer_size = 0;
+			free(item->properties);
+			item->properties = NULL;
+			item->properties_length = 0;
+			item->properties_reserved = 0;
+			microfeed_memory_free(item);
+			item = NULL;;
+			/* TODO: Remove the next. */
+			abort();
+		}
+	}
+
+	return item;
+}
+
 void microfeed_item_free(MicrofeedItem* item) {
 	if (item) {
-		free(item->uid);
 		free(item->properties);
 		free(item->buffer);
-		item->uid = NULL;
 		item->properties = NULL;
 		item->buffer = NULL;
 		microfeed_memory_free(item);
@@ -98,13 +148,6 @@ MicrofeedItem* microfeed_item_duplicate(MicrofeedItem* item) {
 	MicrofeedItem* duplicate;
 	
 	duplicate = microfeed_memory_allocate(MicrofeedItem);
-	if (item->uid) {
-		duplicate->uid = strdup(item->uid);
-	} else {
-		duplicate->uid = NULL;
-	}
-	duplicate->timestamp = item->timestamp;
-	duplicate->status = item->status;
 	duplicate->properties = (size_t*)malloc(item->properties_length * sizeof(size_t));
 	memcpy(duplicate->properties, item->properties, item->properties_length * sizeof(size_t));
 	duplicate->properties_length = duplicate->properties_reserved = item->properties_length;
@@ -115,56 +158,46 @@ MicrofeedItem* microfeed_item_duplicate(MicrofeedItem* item) {
 	return duplicate;	
 }
 
-int microfeed_item_demarshal_properties(MicrofeedItem* item, const void* data, size_t size) {
-	size_t offset;
-
-	if (item->buffer) {
-		free(item->buffer);
-	}
-	item->buffer = (char*)malloc(size);
-	item->buffer_size = size;
-	memcpy(item->buffer, data, size);
-
-	for (offset = 0; offset < size; offset += *((property_size_t*)(item->buffer + offset))) {
-		if (item->properties_length == item->properties_reserved) {
-			item->properties_reserved += 10 * sizeof(size_t);
-			item->properties = (size_t*)realloc(item->properties, item->properties_reserved);
-		}
-		item->properties[item->properties_length++] = offset;
-	}
-	
-	if (offset > size) {
-		free(item->buffer);
-		item->buffer = NULL;
-		item->buffer_size = 0;
-		free(item->properties);
-		item->properties = NULL;
-		item->properties_length = 0;
-		item->properties_reserved = 0;
-	
-		return 0;
-	}
-
-	return 1;
-}
-
 const char* microfeed_item_get_uid(MicrofeedItem* item) {
 
-	return item->uid;
+	return ((Buffer*)item->buffer)->uid;
+}
+
+void microfeed_item_marshalled_get_uid(const void* data, size_t data_size, const char** uid_return, size_t* uid_size_return) {
+	*uid_return = ((Buffer*)data)->uid;
+	*uid_size_return = (size_t)((Buffer*)data)->uid_length + 1;
 }
 
 time_t microfeed_item_get_timestamp(MicrofeedItem* item) {
 	
-	return item->timestamp;
+	return (time_t)ntohl(((Buffer*)item->buffer)->timestamp);
+}
+
+void microfeed_item_marshalled_get_timestamp_uid(const void* data, size_t data_size, const void** timestamp_uid_return, size_t* timestamp_uid_size_return) {
+	*timestamp_uid_return = &((Buffer*)data)->timestamp;
+	*timestamp_uid_size_return = sizeof(uint32_t) + ((Buffer*)data)->uid_length;
 }
 
 void microfeed_item_set_timestamp(MicrofeedItem* item, time_t timestamp) {
-	item->timestamp = timestamp;
+	((Buffer*)item->buffer)->timestamp = htonl((uint32_t)timestamp);
 }
 
 MicrofeedItemStatus microfeed_item_get_status(MicrofeedItem* item) {
 	
-	return item->status;
+	return (MicrofeedItemStatus)(((Buffer*)item->buffer)->status);
+}
+
+MicrofeedItemStatus microfeed_item_marshalled_get_status(const void* data, size_t data_size) {
+
+	return (MicrofeedItemStatus)(((Buffer*)data)->status);
+}
+
+void microfeed_item_set_status(MicrofeedItem* item, MicrofeedItemStatus status) {
+	((Buffer*)item->buffer)->status = (char)status;
+}
+
+void microfeed_item_marshalled_set_status(const void* data, size_t data_size, MicrofeedItemStatus status) {
+	((Buffer*)data)->status = (char)status;
 }
 
 const char* microfeed_item_get_property(MicrofeedItem* item, const char* key) {
@@ -214,9 +247,14 @@ MicrofeedItemIterator* microfeed_item_iterate_properties(MicrofeedItem* item, co
 	return iterator;
 }
 
-void microfeed_item_marshal_properties(MicrofeedItem* item, const void** data_pointer, size_t* size_pointer) {
+void microfeed_item_marshal(MicrofeedItem* item, const void** data_pointer, size_t* data_size_pointer) {
 	*data_pointer = item->buffer;
-	*size_pointer = item->buffer_size;
+	*data_size_pointer = item->buffer_size;
+}
+
+void microfeed_item_marshal_without_properties(MicrofeedItem* item, const void** data_pointer, size_t* data_size_pointer) {
+	*data_pointer = item->buffer;
+	*data_size_pointer = sizeof(Buffer) + ((Buffer*)item->buffer)->uid_length;
 }
 
 void microfeed_item_set_property(MicrofeedItem* item, const char* key, const char* value) {
@@ -224,6 +262,7 @@ void microfeed_item_set_property(MicrofeedItem* item, const char* key, const cha
 		microfeed_item_set_property_full(item, key, strlen(key), value, strlen(value));
 	} else {
 		/* TODO: Remove property. */
+		microfeed_item_set_property_full(item, key, strlen(key), "", 0);
 	}
 }
 
@@ -267,8 +306,8 @@ void microfeed_item_set_property_full(MicrofeedItem* item, const char* key, size
 		(item->buffer + item->properties[index] + sizeof(property_size_t) + key_length + 1)[value_length] = 0;
 	} else {
 		if (item->properties_length == item->properties_reserved) {
-			item->properties_reserved += 10 * sizeof(char*);
-			item->properties = (size_t*)realloc(item->properties, item->properties_reserved);
+			item->properties_reserved += 8 * sizeof(char*);
+			item->properties = (size_t*)realloc(item->properties, item->properties_reserved * sizeof(size_t));
 		}
 
 		buffer_size = item->buffer_size;
