@@ -69,17 +69,23 @@ typedef enum {
 MicrofeedItem* microfeed_item_new_temporary(void);
 MicrofeedItem* microfeed_item_new(const char* uid, time_t timestamp);
 MicrofeedItem* microfeed_item_new_with_status(const char* uid, time_t timestamp, MicrofeedItemStatus status);
+MicrofeedItem* microfeed_item_new_from_marshalled(const void* data, size_t size);
 void microfeed_item_free(MicrofeedItem* item);
 MicrofeedItem* microfeed_item_duplicate(MicrofeedItem* item);
 
-int microfeed_item_demarshal_properties(MicrofeedItem* item, const void* data, size_t size);
 const char* microfeed_item_get_uid(MicrofeedItem* item);
+void microfeed_item_marshalled_get_uid(const void* data, size_t data_size, const char** uid_return, size_t* uid_size_return);
 time_t microfeed_item_get_timestamp(MicrofeedItem* item);
+void microfeed_item_marshalled_get_timestamp_uid(const void* data, size_t data_size, const void** timestamp_uid_return, size_t* timestamp_uid_size_return);
 void microfeed_item_set_timestamp(MicrofeedItem* item, time_t timestamp);
 MicrofeedItemStatus microfeed_item_get_status(MicrofeedItem* item);
+MicrofeedItemStatus microfeed_item_marshalled_get_status(const void* data, size_t data_size);
+void microfeed_item_set_status(MicrofeedItem* item, MicrofeedItemStatus status);
+void microfeed_item_marshalled_set_status(const void* data, size_t data_size, MicrofeedItemStatus status);
 const char* microfeed_item_get_property(MicrofeedItem* item, const char* key);
 MicrofeedItemIterator* microfeed_item_iterate_properties(MicrofeedItem* item, const char* start_key);
-void microfeed_item_marshal_properties(MicrofeedItem* item, const void** data_pointer, size_t* size_pointer);
+void microfeed_item_marshal(MicrofeedItem* item, const void** data_pointer, size_t* data_size_pointer);
+void microfeed_item_marshal_without_properties(MicrofeedItem* item, const void** data_pointer, size_t* data_size_pointer);
 void microfeed_item_set_property(MicrofeedItem* item, const char* key, const char* value);
 void microfeed_item_set_property_with_length(MicrofeedItem* item, const char* key, const char* value, size_t value_length);
 void microfeed_item_set_property_full(MicrofeedItem* item, const char* key, size_t key_length, const char* value, size_t value_length);
