@@ -384,23 +384,16 @@ void microfeed_provider_send_error_signal(MicrofeedProvider* provider, const cha
 	dbus_message_unref(message);	
 }
 
-void microfeed_provider_send_item_data_signal(MicrofeedProvider* provider, const char* destination, const char* object_path, const char* uri, const char* uid, const void* data, size_t data_size) {
+void microfeed_provider_send_data_stored_signal(MicrofeedProvider* provider, const char* destination, const char* object_path, const char* url, const char* filename) {
 	DBusMessage* message;
-	DBusMessageIter iter;
-	DBusMessageIter subiter;
 
-	printf("%s : %s (%s)\n", object_path, MICROFEED_SIGNAL_NAME_ITEM_DATA, destination);
+	printf("%s : %s (%s)\n", object_path, MICROFEED_SIGNAL_NAME_DATA_STORED, destination);
 
-	message = dbus_message_new_signal(object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), MICROFEED_SIGNAL_NAME_ITEM_DATA);
+	message = dbus_message_new_signal(object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), MICROFEED_SIGNAL_NAME_DATA_STORED);
 	if (destination) {
 		dbus_message_set_destination(message,destination);
 	}
-	dbus_message_iter_init_append(message, &iter);
-	dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &uri);
-	dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &uid);
-	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE_AS_STRING, &subiter);
-	dbus_message_iter_append_fixed_array(&subiter, DBUS_TYPE_BYTE, &data, data_size);
-	dbus_message_iter_close_container(&iter, &subiter);
+	dbus_message_append_args(message, DBUS_TYPE_STRING, &url, DBUS_TYPE_STRING, &filename, DBUS_TYPE_INVALID);
 	dbus_connection_send(provider->connection, message, NULL);
 	dbus_message_unref(message);
 }
@@ -450,6 +443,7 @@ void microfeed_provider_remove_timeout(MicrofeedProvider* provider, MicrofeedPro
 
 		microfeed_memory_free(timeout);
 	}
+	microfeed_weak_reference_unref(weak_reference);
 
 	microfeed_object_unlock(provider, MicrofeedProvider);
 }
@@ -578,6 +572,8 @@ static DBusHandlerResult handle_message(DBusConnection* connection, DBusMessage*
 static void start_thread(MicrofeedProvider* provider, MicrofeedProviderTimeoutFunction function, MicrofeedObject* object, void* data) {
 	ThreadData* thread_data;
 
+	microfeed_object_ref(provider, MicrofeedProvider);
+
 	thread_data = microfeed_memory_allocate(ThreadData);
 	thread_data->function = function;
 	thread_data->object = microfeed_object_ref_generic(object);
@@ -604,7 +600,6 @@ static void thread_exited(MicrofeedThread* thread, void* user_data) {
 	
 	provider = (MicrofeedProvider*)user_data;
 
-	microfeed_object_ref(provider, MicrofeedProvider);
 	microfeed_object_lock(provider, MicrofeedProvider);
 	
 	microfeed_store_clean_weak_references(provider->publishers);
@@ -616,6 +611,7 @@ static void thread_exited(MicrofeedThread* thread, void* user_data) {
 	}
 
 	microfeed_object_unlock(provider, MicrofeedProvider);
+	
 	microfeed_object_unref(provider, MicrofeedProvider);
 }	
 
