@@ -41,11 +41,11 @@ struct _MicrofeedPublisher {
 	MicrofeedProvider* provider;
 	MicrofeedPublisherCallbacks callbacks;
 	void* user_data;
-	MicrofeedDatabaseEnvironment* database_environment;
-	
+	MicrofeedDatabase* database;	
 	MicrofeedStore* subscribers;
 	MicrofeedStore* feeds;
-	MicrofeedDatabase* data_database;
+	MicrofeedDatabaseTable* data_table;
+	MicrofeedDatabaseIndex* data_index;
 	MicrofeedStore* store_data_requests;
 };
 
@@ -80,6 +80,11 @@ typedef struct {
 	MicrofeedFeed* feed;
 } UpdateThreadData;
 
+typedef struct {
+	uint64_t reference_count;
+	uint16_t url_length;
+} Data;
+
 static void free_function(MicrofeedPublisher* publisher);
 
 static Subscriber* subscriber_new(MicrofeedPublisher* publisher, const char* unique_connection_name);
@@ -87,6 +92,7 @@ static void subscriber_free(Subscriber* subscriber);
 static const char* subscriber_get_unique_connection_name(Subscriber* subscriber);
 
 static const char* store_data_request_get_url(StoreDataRequest* store_data_request);
+static void data_get_url(Data* data, size_t data_size, const char** url_return, size_t* url_size_return);
 
 static MicrofeedError* method_add_item(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message);
 static MicrofeedError* method_modify_item(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message);
@@ -138,10 +144,10 @@ static MethodCallback method_callbacks[] = {
  * Instantiates a new publisher.
  * 
  * The publisher creates or opens
- * a database environment in the given directory using publisher_identifier as a name.
+ * a database in the given directory using publisher_identifier as a name.
  * 
  * @param publisher_identifier A string containing unique publisher identifier.
- * @param directory A directory where the database environment is created or opened (if exists).
+ * @param directory A directory where the database is created or opened (if exists).
  * @param connection A DBus connection.
  * @param callbacks Functions that are called when a publisher receives a message from a subscriber.
  * @param user_data A pointer to user data that is given as a parameter when a callback function is called.
@@ -174,16 +180,18 @@ MicrofeedPublisher* microfeed_publisher_new(MicrofeedProvider* provider, const c
 			return NULL;
 		}
 		path = microfeed_util_string_concatenate(directory, "/", publisher_identifier, "/db", NULL);
-		if (!(publisher->database_environment = microfeed_database_environment_new("berkeleydb", path))) {
+		if (!(publisher->database = microfeed_database_new("microfeedb", path))) {
 			/* TODO: Free */
 			
 			return NULL;
 		}
 		free(path);
+		
 		publisher->subscribers = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)subscriber_get_unique_connection_name);
 		publisher->feeds = microfeed_store_new_sorted_weak_references((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)microfeed_feed_get_uri);
 
-		publisher->data_database = microfeed_database_environment_get_database(publisher->database_environment, ":microfeed.timestamps", NULL, 1);
+		publisher->data_table = microfeed_database_get_table(publisher->database, "http://microfeed.org/Database/Data");
+		publisher->data_index = microfeed_database_table_get_index(publisher->data_table, "url", 1, (MicrofeedDatabaseIndexFunction)data_get_url, microfeed_database_compare_keys_direct);
 		publisher->store_data_requests = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)store_data_request_get_url);
 	}
 	
@@ -204,8 +212,9 @@ static void free_function(MicrofeedPublisher* publisher) {
 	free(publisher->data_directory);
 	microfeed_store_free(publisher->subscribers);
 	microfeed_store_free(publisher->feeds);
-	microfeed_object_unref(publisher->data_database, MicrofeedDatabase);
-	microfeed_object_unref(publisher->database_environment, MicrofeedDatabaseEnvironment);
+	microfeed_object_unref(publisher->data_index, MicrofeedDatabaseIndex);
+	microfeed_object_unref(publisher->data_table, MicrofeedDatabaseTable);
+	microfeed_object_unref(publisher->database, MicrofeedDatabase);
 	microfeed_store_free(publisher->store_data_requests);
 }
 
@@ -276,9 +285,9 @@ int microfeed_publisher_handle_message(MicrofeedPublisher* publisher, DBusMessag
 }
 
 
-MicrofeedDatabaseEnvironment* microfeed_publisher_get_database_environment(MicrofeedPublisher* publisher) {
+MicrofeedDatabase* microfeed_publisher_get_database(MicrofeedPublisher* publisher) {
 
-	return publisher->database_environment;
+	return publisher->database;
 }
 
 const char* microfeed_publisher_get_object_path(MicrofeedPublisher* publisher) {
@@ -425,7 +434,7 @@ long int microfeed_publisher_get_setting_value_integer(MicrofeedPublisher* publi
 	
 	microfeed_object_lock(publisher, MicrofeedPublisher);
 	
-	feed = get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_SETTINGS,1 );
+	feed = get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_SETTINGS, 1);
 
 	microfeed_object_unlock(publisher, MicrofeedPublisher);
 	
@@ -486,26 +495,29 @@ int microfeed_publisher_set_setting_value(MicrofeedPublisher* publisher, const c
 
 void microfeed_publisher_ref_data(MicrofeedPublisher* publisher, const char* url) {
 	size_t url_size;
-	uint64_t reference_count;
-	size_t size;
+	Data* data;
+	size_t data_size;
 	MicrofeedItem* item;
 	uint64_t timestamp;
 	void* key;
 	size_t key_size;
-	const void* data;
-	size_t data_size;
 	char status;
 
 	microfeed_object_lock(publisher, MicrofeedPublisher);
 
 	url_size = strlen(url) + 1;
-	size = sizeof(uint64_t);
-	if (microfeed_database_get_data_partial(publisher->data_database, url, url_size, &reference_count, &size, 0) && size == sizeof(uint64_t)) {
-		reference_count++;
-		microfeed_database_replace_data_partial(publisher->data_database, url, url_size, &reference_count, size, 0);
+	if (microfeed_database_index_get_data(publisher->data_index, url, url_size, (void**)&data, &data_size)) {
+		data->reference_count++;
+		microfeed_database_index_replace_data(publisher->data_index, data, data_size);
+		free(data);
 	} else {
-		reference_count = 1;
-		microfeed_database_replace_data(publisher->data_database, url, url_size, &reference_count, size);
+		data_size = sizeof(Data) + url_size;
+		data = (Data*)malloc(data_size);
+		data->reference_count = 1;
+		data->url_length = url_size;
+		memcpy(((void*)data) + sizeof(Data), url, url_size);
+		microfeed_database_index_replace_data(publisher->data_index, data, data_size);
+		free(data);
 	}
 
 	microfeed_object_unlock(publisher, MicrofeedPublisher);
@@ -513,28 +525,23 @@ void microfeed_publisher_ref_data(MicrofeedPublisher* publisher, const char* url
 
 void microfeed_publisher_unref_data(MicrofeedPublisher* publisher, const char* url) {
 	size_t url_size;
-	void* data;
+	Data* data;
 	size_t data_size;
-	uint64_t* reference_count_pointer;
 	char* path;
 
 	microfeed_object_lock(publisher, MicrofeedPublisher);
 	
 	url_size = strlen(url) + 1;
-	data_size = sizeof(uint64_t);
-	if (microfeed_database_get_data(publisher->data_database, url, url_size, &data, &data_size)) {
-		if (data_size >= sizeof(uint64_t)) {
-			reference_count_pointer == (uint64_t*)data;
-			(*reference_count_pointer)--;
-			if (*reference_count_pointer == 0) {
-				path = microfeed_util_string_concatenate(publisher->data_directory, data + sizeof(uint64_t), NULL);
-				unlink(path);
-				free(path);
+	if (microfeed_database_index_get_data(publisher->data_index, url, url_size, (void**)&data, &data_size)) {
+		data->reference_count--;
+		if (data->reference_count == 0) {
+			path = microfeed_util_string_concatenate(publisher->data_directory, ((void*)data) + sizeof(Data) + data->url_length, NULL);
+			unlink(path);
+			free(path);
 
-				microfeed_database_remove_data(publisher->data_database, url, url_size);
-			} else {
-				microfeed_database_replace_data_partial(publisher->data_database, url, url_size, data, sizeof(uint64_t), 0);
-			}
+			microfeed_database_index_remove_data(publisher->data_index, url, url_size);
+		} else {
+			microfeed_database_index_replace_data(publisher->data_index, data, data_size);
 		}
 		free(data);
 	}
@@ -874,9 +881,6 @@ static MicrofeedError* method_subscribe_feed(MicrofeedPublisher* publisher, Subs
 	char* uri;
 	DBusMessage* reply;
 	MicrofeedItem* item;
-	unsigned long int interval;
-	long last_time = 0;
-	time_t elapsed;
 
 	dbus_error_init(&error);
 	if (!dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_INVALID)) {
@@ -906,18 +910,10 @@ printf(" SUBCRIBE: %s(%p) %p %s\n", microfeed_feed_get_uri(feed), feed, publishe
 	microfeed_object_unlock(publisher, MicrofeedPublisher);
 
 	microfeed_feed_send_metadata_item(feed, subscriber->unique_connection_name);
-	interval = (unsigned long int)microfeed_publisher_get_setting_value_integer(publisher, "update_interval", 0) * 60;
+	microfeed_feed_start_periodical_update(feed);
 	
 	microfeed_object_lock(publisher, MicrofeedPublisher);
 	
-	if (interval) {
-		last_time = microfeed_feed_get_last_updated_time(feed);
-printf("LAST UPDATE: %s %s %ld\n", publisher->identifier, uri, last_time);
-
-		elapsed = time(NULL) - last_time;
-		microfeed_provider_add_timeout(publisher->provider, (elapsed > interval ? 0 : interval - elapsed), interval, (MicrofeedProviderTimeoutFunction)microfeed_feed_update, microfeed_object_get_weak_reference(feed, MicrofeedFeed), NULL);
-	}
-	
 	microfeed_object_unref(feed, MicrofeedFeed);
 
 	return (MicrofeedError*)-1;
@@ -1012,7 +1008,7 @@ static MicrofeedError* method_store_data(MicrofeedPublisher* publisher, Subscrib
 	DBusError dbus_error;
 	const char* url;
 	size_t url_size;
-	void* data;
+	Data* data;
 	size_t data_size;
 	char* path;
 	StoreDataRequest* store_data_request;
@@ -1033,12 +1029,12 @@ static MicrofeedError* method_store_data(MicrofeedPublisher* publisher, Subscrib
 		
 		return microfeed_error_new(MICROFEED_ERROR_DATA_NOT_SUPPORTED, "Publisher does not support storing data");
 	}
-	if (!microfeed_database_get_data(publisher->data_database, url, url_size, &data, &data_size)) {
+	if (!microfeed_database_index_get_data(publisher->data_index, url, url_size, (void**)&data, &data_size)) {
 
 		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_DATA, "Requested data does not exist");
 	}
-	if (data_size > sizeof(uint64_t)) {
-		path = microfeed_util_string_concatenate(publisher->data_directory, data + sizeof(uint64_t), NULL);
+	if (data_size > sizeof(Data) + url_size) {
+		path = microfeed_util_string_concatenate(publisher->data_directory, ((void*)data) + sizeof(Data) + data->url_length, NULL);
 		if (microfeed_util_file_exists(path)) {
 			microfeed_provider_send_data_stored_signal(publisher->provider, subscriber->unique_connection_name, publisher->object_path, url, data + sizeof(uint64_t));
 			free(data);
@@ -1046,7 +1042,7 @@ static MicrofeedError* method_store_data(MicrofeedPublisher* publisher, Subscrib
 			
 			return NULL;
 		} else {
-			microfeed_database_replace_data(publisher->data_database, url, url_size, data, sizeof(uint64_t));
+			microfeed_database_index_replace_data(publisher->data_index, data, sizeof(Data) + data->url_length);
 		}
 		free(path);
 	}
@@ -1074,18 +1070,18 @@ static MicrofeedError* method_store_data(MicrofeedPublisher* publisher, Subscrib
 
 		microfeed_util_file_create(path, stored_data, stored_data_size);
 
-		data_size = sizeof(uint64_t) + strlen(name) + 1;
+		data_size = sizeof(Data) + url_size + strlen(name) + 1;
 		data = realloc(data, data_size);
-		memcpy(data, name, data_size - sizeof(uint64_t));
-		microfeed_database_replace_data(publisher->data_database, url, url_size, data, data_size);
+		memcpy(data, name, data_size - sizeof(Data) - url_size);
+		microfeed_database_index_replace_data(publisher->data_index, data, data_size);
 
 		microfeed_provider_send_data_stored_signal(publisher->provider, NULL, publisher->object_path, url, name);
 
 		free(name);
 		free(path);
+		free(stored_data);
 	}
 
-	free(stored_data);
 	free(data);
 
 	microfeed_store_remove(publisher->store_data_requests, store_data_request);
@@ -1184,6 +1180,10 @@ static MicrofeedError* settings_modify_item(MicrofeedFeed* feed, MicrofeedItem*
 	MicrofeedError* error = NULL;;
 	MicrofeedPublisher* publisher;
 	const char* value;
+	MicrofeedStoreIterator* subscriber_iterator;
+	MicrofeedStoreIterator* feed_iterator;
+	Subscriber* subscriber;
+	MicrofeedFeed* subscribed_feed;
 	
 	publisher = (MicrofeedPublisher*)user_data;
 	
@@ -1195,7 +1195,21 @@ static MicrofeedError* settings_modify_item(MicrofeedFeed* feed, MicrofeedItem*
 				microfeed_item_set_property(existing_item, MICROFEED_ITEM_PROPERTY_NAME_SETTING_VALUE, value);
 				microfeed_feed_replace_item(feed, existing_item);
 			}
-			/* TODO: If internal setting (such as update_interval), do necessary actions. */
+
+			/* If internal setting (such as update_interval), do necessary actions. */
+			if (!strcmp(microfeed_item_get_uid(new_item), "update_interval")) {
+				for (subscriber_iterator = microfeed_store_iterate(publisher->subscribers, NULL);
+				     (subscriber = microfeed_store_iterator_get(subscriber_iterator, Subscriber));
+				     microfeed_store_iterator_next(subscriber_iterator)) {
+					for (feed_iterator = microfeed_store_iterate(subscriber->feeds, NULL);
+					     (subscribed_feed = microfeed_store_iterator_get(feed_iterator, MicrofeedFeed));
+					     microfeed_store_iterator_next(feed_iterator)) {
+						microfeed_feed_start_periodical_update(subscribed_feed);
+					}
+					microfeed_store_iterator_free(feed_iterator);
+				}
+				microfeed_store_iterator_free(subscriber_iterator);
+			}
 		} else {
 			error = microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "No setting.value property in settings item.");
 		}
@@ -1203,3 +1217,8 @@ static MicrofeedError* settings_modify_item(MicrofeedFeed* feed, MicrofeedItem*
 	
 	return error;
 }
+
+static void data_get_url(Data* data, size_t data_size, const char** url_return, size_t* url_size_return) {
+	*url_return = ((void*)data) + sizeof(Data);
+	*url_size_return = data->url_length;
+}
