@@ -119,7 +119,7 @@ typedef struct _MicrofeedPublisherCallbacks {
 
 MicrofeedPublisher* microfeed_publisher_new(MicrofeedProvider* provider, const char* unique_identifier, const char* directory, MicrofeedPublisherCallbacks* callbacks, void* user_data);
 int microfeed_publisher_handle_message(MicrofeedPublisher* publisher, DBusMessage* message);
-MicrofeedDatabaseEnvironment* microfeed_publisher_get_database_environment(MicrofeedPublisher* publisher);
+MicrofeedDatabase* microfeed_publisher_get_database(MicrofeedPublisher* publisher);
 const char* microfeed_publisher_get_object_path(MicrofeedPublisher* publisher);
 const char* microfeed_publisher_get_identifier(MicrofeedPublisher* publisher);
 MicrofeedProvider* microfeed_publisher_get_provider(MicrofeedPublisher* publisher);
