@@ -202,12 +202,25 @@ MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, DBusConnec
 void microfeed_subscriber_free(MicrofeedSubscriber* subscriber) {
 	char buffer[1024];
 
+	dbus_connection_unregister_object_path(subscriber->connection, subscriber->object_path);
+	dbus_connection_remove_filter(subscriber->connection, object_message, subscriber);
+
 	snprintf(buffer, 1024, "type='signal',interface='%s',destination='%s'", MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(subscriber->connection));
 	dbus_bus_remove_match(subscriber->connection, buffer, NULL);
 	snprintf(buffer, 1024, "type='signal',interface='%s',destination ='%s'", MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(subscriber->connection));
 	dbus_bus_remove_match(subscriber->connection, buffer, NULL);
 
-	/* TODO */
+	free(subscriber->identifier);
+	free(subscriber->object_path);
+	microfeed_configuration_free(subscriber->configuration);
+	/* TODO: Free items in store. */
+	microfeed_store_free(subscriber->providers_by_bus_name);
+	/* TODO: Free items in store. */
+	microfeed_store_free(subscriber->providers_by_unique_connection_name);
+	/* TODO: Free items in store. */
+	microfeed_store_free(subscriber->publishers_by_identifier);
+
+	microfeed_memory_free(subscriber);
 }
 
 const char* microfeed_subscriber_get_identifier(MicrofeedSubscriber* subscriber) {
@@ -1196,14 +1209,19 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 					if (feed->updating && s->callbacks->feed_update_ended) {
 						s->callbacks->feed_update_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
 					}
+/*
 					if (s->callbacks->error_occured) {
-						s->callbacks->error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", s->user_data);	
+						s->callbacks->error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", s->user_data);
 					}
+ Instead of error we are now resubscribing the feed.
+*/
+					message = dbus_message_new_method_call(feed->publisher->provider->bus_name, feed->publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_SUBSCRIBE_FEED);
+					dbus_message_append_args(message, DBUS_TYPE_STRING, &feed->uri, DBUS_TYPE_INVALID);
+					call_publisher_method(publisher, uri, NULL, message, s->callbacks->error_occured, user_data);
+					dbus_message_unref(message);
 				}
-				/* TODO: Think! Think! Think! feed_free(feed); */
 			}
 			microfeed_store_iterator_free(feed_iterator);
-			/* TODO: I am quite sure that this should not be here: publisher_unref(publisher); */
 		}
 		provider_set_unique_connection_name(provider, NULL);
 		microfeed_store_iterator_free(publisher_iterator);
