@@ -61,7 +61,7 @@ typedef void (*MicrofeedSubscriberFeedCallback)(MicrofeedSubscriber* subscriber,
 typedef void (*MicrofeedSubscriberItemCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, MicrofeedItem* item, void* user_data);
 typedef void (*MicrofeedSubscriberItemRemovedCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, void* user_data);
 typedef void (*MicrofeedSubscriberStatusCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const MicrofeedItemStatus status, void* user_data);
-typedef void (*MicrofeedSubscriberItemDataCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, const char* uid, const void* data, const size_t length, void* user_data);
+typedef void (*MicrofeedSubscriberDataCallback)(MicrofeedSubscriber* subscriber, const char* publisher, const char* url, const char* path, void* user_data);
 
 typedef struct {
 	MicrofeedSubscriberErrorCallback error_occured;
@@ -74,7 +74,6 @@ typedef struct {
 	MicrofeedSubscriberItemCallback item_republished;
 	MicrofeedSubscriberItemRemovedCallback item_removed;
 	MicrofeedSubscriberStatusCallback item_status_changed;
-	MicrofeedSubscriberItemDataCallback item_data_received;
 } MicrofeedSubscriberCallbacks;
 
 typedef void (*MicrofeedSubscriberConfiguredSubscribeCallback)(MicrofeedSubscriber* subscriber, const char* publisher, void* callback_data);
@@ -96,7 +95,11 @@ int microfeed_subscriber_republish_items(MicrofeedSubscriber* subscriber, const
 int microfeed_subscriber_get_item_data(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_mark_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_unmark_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
-int microfeed_subscriber_read_items(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* start_uid, const char* end_uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
+int microfeed_subscriber_read_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
+
+int microfeed_subscriber_store_data(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* url, MicrofeedSubscriberErrorCallback callback, void* user_data);
+void microfeed_subscriber_add_data_stored_callback(MicrofeedSubscriber* subscriber, const char* publisher_identifier, MicrofeedSubscriberDataCallback callback, void* user_data);
+void microfeed_subscriber_remove_data_stored_callback(MicrofeedSubscriber* subscriber, const char* publisher_identifier, MicrofeedSubscriberDataCallback callback, void* user_data);
 
 int microfeed_subscriber_create_publisher(MicrofeedSubscriber* subscriber, const char* publisher, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_destroy_publisher(MicrofeedSubscriber* subscriber, const char* publisher, MicrofeedSubscriberReplyCallback callback, void* user_data);
